//############################################################################
//
// COMPANY:	Pera Technology 2012
//
// FILE:   Alert_main.c
//
// TITLE:  DSP280xx SD Card SPI Interface for Alert Project 
//
// ASSUMPTIONS:
//
//    This program uses the DSP280xx header files already included in the  
// 	  download. 
//
//	  An SD Card interface to SPI is also required to properly test
//	  this example. A reference design is supplied in the Application Note:
//	 "Interfacing SD/MMC Flash Memory Cards with F28xxx."
//
// DESCRIPTION:
//	 
//	Alert_SD_SCI_Dual_Laser carries out the following functions:
//	1)	Initialises peripherals such as the SD card, the LCD, interrupts and the ADC registers.
//	2)	Waits in an idle condition for an interrupt either from the user or Photo1 or Photo2.
//		A user interrupt will either reset the system from SW1 or turn on the backlight for the LCD, SW2.
//		An interrupt from Photo1 or Photo2 indicates the detection of a particle from one of the diodes.
//		If the light intensity if sufficient as measured by ADCRESULT0, the linear arrays are read.
//		Following this the algorithm is processed to determine the statiistics of the detection.
//	This project writes to an SD/MMC Memory Card. It checks for card insertion,
//	initializes the card, reads card registers, Read, Write, and Erase Data from 
//	SD/MMC Memory Card. 
//	
//############################################################################		
// Original Author: Tim Love			SD Card Media Read/Write
//					Elm Chang			FAT32 Support 
// 					Brendan Devaney		Main programme and algorithm
//					Geoff Lawson		SCI output, ADC processing, 
//										SPI initialisation, Dual Laser code
//										FAT file support for SD
// Release Date: 						Version 1.1
//										Version 1.2
//										Version 1.3 04/10/2012
//										Version 1.4	25/10/2012
//										Version 1.5 13/11/2012	1	ISR's run from SARAM for faster clocking of arrays at 4.38MHz.
//																2	Removes background noise from arrays.
//																3	Reverses array2 data for correct alignment of peaks.
//																4	Date/ time written to CSV file.
//										Version 1.6	26/11/2012	1	Added memcopy code to place interrupt vector table in RAM.
//																	This enables the interrupts to be accessed without debugger.
//############################################################################


/* ***********************************************************
* THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
* REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
* FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
* COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
* TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
* POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
* INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
* YOUR USE OF THE PROGRAM.
*
* IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
* CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
* THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
* OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
* OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
* EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
* REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
* OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
* USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
* AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
* YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
* (U.S.$500).
*
* Unless otherwise stated, the Program written and copyrighted 
* by Texas Instruments is distributed as "freeware".  You may, 
* only under TI's copyright in the Program, use and modify the 
* Program without any charge or restriction.  You may 
* distribute to third parties, provided that you transfer a 
* copy of this license to the third party and the third party 
* agrees to these terms by its first use of the Program. You 
* must reproduce the copyright notice and any other legend of 
* ownership on each copy or partial copy, of the Program.
*
* You acknowledge and agree that the Program contains 
* copyrighted material, trade secrets and other TI proprietary 
* information and is protected by copyright laws, 
* international copyright treaties, and trade secret laws, as 
* well as other intellectual property laws.  To protect TI's 
* rights in the Program, you agree not to decompile, reverse 
* engineer, disassemble or otherwise translate any object code 
* versions of the Program to a human-readable form.  You agree 
* that in no event will you alter, remove or destroy any 
* copyright notice included in the Program.  TI reserves all 
* rights not specifically granted under this license. Except 
* as specifically provided herein, nothing in this agreement 
* shall be construed as conferring by implication, estoppel, 
* or otherwise, upon you, any license or other right under any 
* TI patents, copyrights or trade secrets.
*
* You may not use the Program in non-TI devices.
* ********************************************************* */

#include "SD.h"
#include "Alert_Globals.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "DSP28x_Project.h"
#include "cmdline.h"
#include "ff_ff9a.h"


//*******************************************************************************
//							 	Internal functions
//*******************************************************************************
extern int _printfi(char const **_format, va_list _ap,     
         void *_op, int (*_outc)(char c, void *op),
         int (*_outs)(char *s, void *op));

void uart_putc( int c )
{
     // send character directly to UARTA or send it to a buffer
     // to decouple character output
     while (SciaRegs.SCIFFTX.bit.TXFFST != 0) {}   
    SciaRegs.SCITXBUF=c;                                           
}
void uart_puts( const char *s )
{
     while ( *s != '\0') {
         uart_putc( (char)*(s++) );
     }
}   // uart_puts 

static int _outc( char c, void *_op )
{
     uart_putc( (char)c );
     return 1;
}   // _outc
static int _outs( char *s, void *_op )
{
     size_t len = strlen( s );
     uart_puts( s );
     return (int)len;
}   // _outs
void uart_printf( const char *fmt, ... ) 
{
     va_list _ap;
     va_start( _ap, fmt );
     _printfi( &fmt, _ap, NULL, _outc, _outs);            
     va_end( _ap );
}   // uart_printf 

//----- End code ---------------------------------------



// *****************************************************************************
// *****************************************************************************
// Internal Function Prototypes
// *****************************************************************************
// *****************************************************************************

void spi_fifo_init(void);

interrupt void xint3_isr(void);						/* 1 second square wave interrupt */
interrupt void cpu_timer0_isr(void);				/* timer0 used to record time of flight between photo diode1 and photo diode2 interrupts.*/
interrupt void cpu_timer1_isr(void);
interrupt void cpu_timer2_isr(void);
interrupt void b_l_isr(void);
interrupt void alm_rst_isr(void);
interrupt void photo1_trigger_isr(void);			/* interrupt from photo diode1 first diode beam */
interrupt void photo2_trigger_isr(void);			/* interrupt from photo diode2 second diode beam */
  
void scia_echoback_init(void);  					/* serial port printf functions */ 
void scia_fifo_init(void);   
void scia_xmit(int a);   
void scia_msg(char *msg);   

//******************************************************************************
// Macros
// *****************************************************************************
// *****************************************************************************

void die (      /* Stop with dying message */
    FRESULT fresult  /* FatFs return value */
)
{
    uart_printf("Failed with fresult=%u.\n", fresult);
  
}
// Functions that will be run from RAM need to be assigned to
// a different section.  This section will then be mapped using
// the linker cmd file.
#pragma CODE_SECTION(photo1_trigger_isr, "ramfuncs");
#pragma CODE_SECTION(photo2_trigger_isr, "ramfuncs");



// These are defined by the linker (see F28335.cmd)
extern Uint16 RamfuncsLoadStart;
extern Uint16 RamfuncsLoadEnd;
extern Uint16 RamfuncsRunStart;
// *****************************************************************************
// 								Global Variables
// *****************************************************************************
// *****************************************************************************
 const int threshold_angle = 10;/* Tests for a fibre orientation change of greater than 10 degrees */
 char version = 1;
 char build = 6;				/* dual laser interrupts added */
 								/* ADC modified */
 								/* CSV file data written to SD */
 								/* unused interrupts disabled */
 								/* Removes background noise from arrays */
 								/* Revserses Array2 contents only for correct data readings */ 
 								/* Formatted timestamp to SD CSV file */
 								/* ISR's now in RAM clocks the arrays at 4.38MHz */
 								/* 'memcopy' code added to ensure interrupts run without debugger. */
 
FATFS Fatfs;        			/* File system object */
FIL File;           			/* File object */
BYTE Buff[128];     			/* File read buffer */


// ADC start parameters
#if (CPU_FRQ_150MHZ)     // Default - 150 MHz SYSCLKOUT
  #define ADC_MODCLK 0x3 // HSPCLK = SYSCLKOUT/2*ADC_MODCLK2 = 150/(2*3)   = 25.0 MHz
#endif
#if (CPU_FRQ_100MHZ)
  #define ADC_MODCLK 0x2 // HSPCLK = SYSCLKOUT/2*ADC_MODCLK2 = 100/(2*2)   = 25.0 MHz
#endif



//################################## MAIN ####################################
int main(void)
{
	int t;
	int nStatus;
	int WriteCount;
    FRESULT fresult;  		/* Result code */ 
   	DIR dir;                /* Directory object */
    FILINFO fno;            /* File information object */
    UINT bw, br;
    unsigned int file_number_int = 1000;
    unsigned int file_number = 0;
    char file_number_str[] =  "ALERT100"; 
    char file_ext_str[] = ".CSV";
    
	
	/* Initialize the on-chip flash registers */
	/*InitFlash();*/							   
// 	Step 1. Initialize System Control:
//	PLL, WatchDog, enable Peripheral Clocks
// 	This example function is found in the DSP280x_SysCtrl.c file.
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);
   	InitSysCtrl();
 	EALLOW;
   	#if (CPU_FRQ_150MHZ)     							// Default - 150 MHz SYSCLKOUT
    #define ADC_MODCLK 0x3 								// HSPCLK = SYSCLKOUT/2*ADC_MODCLK2 = 150/(2*3)   = 25.0 MHz
   	#endif
   	#if (CPU_FRQ_100MHZ)
    #define ADC_MODCLK 0x2 								// HSPCLK = SYSCLKOUT/2*ADC_MODCLK2 = 100/(2*2)   = 25.0 MHz
   	#endif
   	SysCtrlRegs.HISPCP.all = ADC_MODCLK;    			// HSPCLK = SYSCLKOUT/ADC_MODCLK                  <=== ADD THIS LINE
   	EDIS;  	
   	
//	Step 2. Initalize GPIO: 
// 	This example function is found in the DSP280x_Gpio.c file and
// 	illustrates how to set the GPIO to it's default state.
 	InitGpio();
// 	Setup only the GP I/O only for SPI-A functionality
// 	This function is found in DSP280x_Spi.c
   	InitSpiaGpio();
// 	Setup the serial communication interface in DSP2833x_SCI.c
	InitSciaGpio();
// 	Step 3. Clear all interrupts and initialize PIE vector table:
// 	Disable CPU interrupts 
   	DINT;

// 	Initialize PIE control registers to their default state.
// 	The default state is all PIE interrupts disabled and flags
// 	are cleared.  
// 	This function is found in the DSP280x_PieCtrl.c file.
   	InitPieCtrl();

// 	Disable CPU interrupts and clear all CPU interrupt flags:
   	IER = 0x0000;
   	IFR = 0x0000;
   
// 	Initialize the PIE vector table with pointers to the shell Interrupt 
// 	Service Routines (ISR).  
// 	This will populate the entire table, even if the interrupt
// 	is not used in this example.  This is useful for debug purposes.
// 	The shell ISR routines are found in DSP280x_DefaultIsr.c.
// 	This function is found in DSP280x_PieVect.c.
   	InitPieVectTable();
   	
	EALLOW;
	PieVectTable.XINT3 = &xint3_isr;					//edited out disable RTCC for debugging G Lawson
	PieVectTable.TINT2 = &cpu_timer2_isr;
	PieVectTable.XINT13 = &cpu_timer1_isr;
	PieVectTable.XINT4 = &photo2_trigger_isr;			/* map XINT4 to photo2_trigger_isr */
	PieVectTable.XINT5 = &alm_rst_isr;
	PieVectTable.XINT6 = &photo1_trigger_isr;			/* map XINT6 to photo1_trigger_isr */
	/*PieVectTable.XINT7 = &photo2_trigger_isr;*/		// map XINT7 to photo2_trigger_isr
	EDIS;
	
	InitCpuTimers();
	// Configure CPU-Timer 0 to interrupt every microsecond:
   	// 150MHz CPU Freq, 1 Period (in uSeconds)
   	// Generates a 100us period 
	ConfigCpuTimer(&CpuTimer0, 150, 50);				// configure timer0 to interrupt every 1 microsecond for testing purposes only
														// timer0 enabled by Photo1_Trigger_ISR and disabled by Photo2_Trigger_ISR
	ConfigCpuTimer(&CpuTimer1, 150, 500);				// configure timer1 for 1000us period
	ConfigCpuTimer(&CpuTimer2, 150, 165000);			// configure timer2 for 330000us period
	
	I2CA_Init();										
// 	Step 4. Initialize all the Device Peripherals:	
   	InitAdc();  										// For this example, init the ADC
   	spi_initialization();								// Initialize SPI
   	
// 	Enable ADCINT in PIE
   	PieCtrlRegs.PIEIER1.bit.INTx6 = 1;
   	IER |= M_INT1; 										// Enable CPU Interrupt 1
   	EINT;          										// Enable Global interrupt INTM
   	ERTM;          										/* Enable Global realtime interrupt DBGM */	   	
   	
// 	Step 5. User specific code:	
// 	Enable external interrupts in the PIE: Group 12 interrupt 3, 4, 5 & 6
// 	Enable int1 which is connected to WAKEINT:
   	PieCtrlRegs.PIECTRL.bit.ENPIE = 1;          		// Enable the PIE block
   	PieCtrlRegs.PIEIER12.bit.INTx1 = 1;
   	PieCtrlRegs.PIEIER12.bit.INTx2 = 1;
   	PieCtrlRegs.PIEIER12.bit.INTx3 = 1;  				//edited out for debugging G Lawson
    PieCtrlRegs.PIEIER12.bit.INTx4 = 1;					/* Enable photo2_trigger interrupt */
      
   	IER |= M_INT1 | M_INT12;
   	IER |= M_INT13 | M_INT14; 

   	PieCtrlRegs.PIEIER1.bit.INTx7 = 1;
   	EINT;                                       		// Enable Global Interrupts
   	ERTM;   											// Enable Global realtime interrupt DBGM

// 	Configure interrupt sampling and qualification 
   	EALLOW;
   	GpioCtrlRegs.GPBMUX1.bit.GPIO34 = 0;         		// 1 second tick
   	GpioCtrlRegs.GPBDIR.bit.GPIO34 = 0;          		// input
   	GpioCtrlRegs.GPBQSEL1.bit.GPIO34 = 2;        		// XINT3 Qual using 6 samples
   	GpioCtrlRegs.GPBCTRL.bit.QUALPRD0 = 0xFF;    		// Sampling window is 510*SYSCLKOUT

   	GpioCtrlRegs.GPBQSEL2.bit.GPIO48 = 2;				// Alarm reset switch
  // GpioCtrlRegs.GPBQSEL2.bit.GPIO49 = 2;				// Back-light switch now used for P_Diode2 interrupt
   	GpioCtrlRegs.GPBCTRL.bit.QUALPRD2 = 0xFF;    		// Sampling window is 510*SYSCLKOUT  
   
   	GpioCtrlRegs.GPBQSEL2.bit.GPIO61 = 2;				// P_Diode1 input
   	GpioCtrlRegs.GPBCTRL.bit.QUALPRD3 = 0x0A;    		// Sampling window is 14*SYSCLKOUT
   
   	GpioCtrlRegs.GPBQSEL2.bit.GPIO49 = 2;				// P_Diode2 input G.Lawson added 10/02/12
   	GpioCtrlRegs.GPBCTRL.bit.QUALPRD2 = 0x0A;    		// Sampling window is 14*SYSCLKOUT
   	
  	EDIS;

/* Assign GPIO pins as external interrupts */
/* XINT1, XINT2 and XNMI accept interrupts on GPIO0 to GPIO31 */
/* XINT3 to XINT7 accept interrupts on GPIO32 to GPIO63 */
 	EALLOW;
   	GpioIntRegs.GPIOXINT3SEL.bit.GPIOSEL = 0x22;		/* XINT3 is 1 second tick (GPIO34)  G Lawson */
    GpioIntRegs.GPIOXINT4SEL.bit.GPIOSEL = 0x31;		/* XINT4 is back-light switch (GPIO49) now use for P_DIODE2 interrupt */
   	GpioIntRegs.GPIOXINT5SEL.bit.GPIOSEL = 0x30;		/* XINT5 is alarm reset switch (GPIO48) */
   	GpioIntRegs.GPIOXINT6SEL.bit.GPIOSEL = 0x3D;		/* XINT6 is photo-diode1 trigger (GPIO61) */  
   	GpioIntRegs.GPIOXINT7SEL.bit.GPIOSEL = 0x31;		/* XINT7 is photo-diode2 trigger (GPIO49)  G.Lawson added 30/04/12 */
       
   	EDIS;
// Interrupt Edge Polarity
   	XIntruptRegs.XINT3CR.bit.POLARITY = 1;      		/* Rising edge interrupt GL */
   	XIntruptRegs.XINT4CR.bit.POLARITY = 1;      		/* Rising edge interrupt */
   	XIntruptRegs.XINT5CR.bit.POLARITY = 1;      		/* Rising edge interrupt */
   	XIntruptRegs.XINT6CR.bit.POLARITY = 1;      		/* Rising edge interrupt */
   	XIntruptRegs.XINT7CR.bit.POLARITY = 1;      		/* Rising edge interrupt */
   
/* Enable CPU INT8 which is connected to PIE group 8 */
   	IER |= M_INT8 | M_INT12;
   	EINT;
   
    I2C_Init();  											//edited out for debugging G Lawson
	GpioDataRegs.GPBCLEAR.bit.GPIO50 = 1;   				// Configure GPIO50 for Red Alarm
//	GpioDataRegs.GPBSET.bit.GPIO54 = 1;     				// Configure GPIO54 for Amber Status
	GpioDataRegs.GPBCLEAR.bit.GPIO51 = 1;   				// Configure GPIO51 for Amber Alarm
//	GpioDataRegs.GPBSET.bit.GPIO55 = 1;     				// Configure GPIO55 for Green Status
	GpioDataRegs.GPBCLEAR.bit.GPIO52 = 1;   				// Configure GPIO52 for Green Alarm

	GpioDataRegs.GPBSET.bit.GPIO35 = 0;     				// Configure GPIO35 for Buzzer O/P
	GpioDataRegs.GPBCLEAR.bit.GPIO35 = 1;   				// Configure GPIO35 for Buzzer O/P

//	GpioDataRegs.GPBSET.bit.GPIO56 = 1;     				/* Configure GPIO56 for B/L Adjust */
	GpioDataRegs.GPBSET.bit.GPIO53 = 1;     				/* Configure GPIO53 for Red Status */
	
	InitLCD();
	
/* Serial Communicatiuon Interface intialised here */
/* Set for 115,200 baud, 8 bits, 1 stop bit and no parity */
	scia_fifo_init();      									/* Initialize the SCI FIFO */    
    scia_echoback_init();  									/* Initalize SCI for echoback */  
  
    uart_printf("\r\nAlert V%d.%d\r\n",version,build); 
    
   
	/* Enables */
   	XIntruptRegs.XINT3CR.bit.ENABLE = 0;  					/* XINT3 is 1 second tick (GPIO34)*/
   	XIntruptRegs.XINT4CR.bit.ENABLE = 1;					/* XINT4 is photo-diode2 trigger (GPIO49) */
   	XIntruptRegs.XINT5CR.bit.ENABLE = 0;					/* XINT5 is alarm reset switch (GPIO48) */
   	XIntruptRegs.XINT6CR.bit.ENABLE = 1;					/* XINT6 is photo-diode1 trigger (GPIO61) */  
   /*	XIntruptRegs.XINT7CR.bit.ENABLE = 1;*/					/* XINT7 is photo-diode2 trigger (GPIO49) */
 	RESET1_OFF;
 	RESET2_OFF;
 
 /**********************************************************/
 						/* SD here */ 	
   	char st1[12] = "/Alert5";
   	char st2[5] = ".txt";
 
	strcat(st1,st2);
	uart_printf("%s\n",st1);
 /* Register work area for each volume (Always succeeds regardless of disk status) */
    fresult = f_mount(0, &Fatfs);							/* Register volume work area (never fails) */ 
    if (fresult) die(fresult);   
/* Open source file on the drive 0 */
	
    fresult = f_open(&File, "/Alert.csv", FA_OPEN_EXISTING | FA_READ); 
	if (fresult) die(fresult);   
	uart_printf("Alert.csv contents\r\n");
for(;;) 
	{ 
		fresult = f_read(&File,Buff,sizeof Buff,&br);			/* read file */
		if (fresult || !br) break; 
		uart_printf(Buff);
	} 
	fresult = f_close(&File);									/* close file */
	if (fresult) die(fresult);  
	
 	uart_printf("\nCreate a new file %s.\n",st1);
 	I2C_datetime_read();
 	//uart_printf("Date %d-%d-%d\r\n",date_uc,month_uc,year_uc);
 	//uart_printf("Time %2d:%2d:%2d\r\n",&hours_uc,&minutes_uc,&seconds_uc);
 	
 	fresult = f_mount(0, &Fatfs);
   	fresult = f_open(&File, "/ALERT8.CSV", FA_WRITE | FA_CREATE_ALWAYS);
    if (fresult) die(fresult); 
    I2C_datetime_read();  
    WriteCount = f_printf(&File, "Alert, v%D,b%D\r\n",version,build);		
    WriteCount = f_printf(&File, "Date,%D-%D-%D\r\n",date_uc,month_uc,year_uc);
    WriteCount = f_printf(&File, "Time,%D:%D:%D\r\n",hours_uc,minutes_uc,seconds_uc);
    uart_printf("%d bytes written.\r\n", bw);

    uart_printf("\nClose the file.\n");
    
 	fresult = f_sync(&File);		
    fresult = f_close(&File);
    /* Unregister work area prior to discard it */
    f_mount(0, NULL);
   
 /******************************************************************************************/  
  
  	
   	/* Interrupts that are used in this example are re-mapped to
	   ISR functions found within this file. */
   EALLOW;  												/* This is needed to write to EALLOW protected registers */
   PieVectTable.TINT0 = &cpu_timer0_isr;
   EDIS;    												/* This is needed to disable write to EALLOW protected registers */
   
	/* To ensure precise timing, use write-only instructions to write to the entire register. Therefore, if any
	  of the configuration bits are changed in ConfigCpuTimer and InitCpuTimers (in DSP2833x_CpuTimers.h), the
	  below settings must also be updated. */
	
	CpuTimer0Regs.TCR.all = 0x4001; 						/* Use write-only instruction to set TSS bit = 0 */
	
/******************************************************************************************/  
	/*Read_Linear_Array_Backgrounds(); */ 						/* Read the dark light voltage levels */
															/* on each of the arrays as a noise reference */
															/* Subtract this level from all further readings */	

	PUMP_ON;
	 // asm(" ESTOP0");										/* idle mode */
   	while(1)
   	{
   		EALLOW;
   		SysCtrlRegs.WDKEY = 0x55;  							/* service watchdog #1 */
   		 //asm(" IDLE");									/* idle mode 100mA */
   		  asm("STANDBY");									/* standy mode 8mA */
   		
   		

   		if (ISR1_flag || ISR2_flag)							/* do this if either ISR's  */
   		{
   			EDIS;											/* Disable global interrupts */										
   			PieCtrlRegs.PIEIER12.bit.INTx4 = 0;				/* Disable photo2_trigger interrupt */
   			PieCtrlRegs.PIEIER12.bit.INTx6 = 0;				/* Disable photo1_trigger interrupt */
   			asm ("      nop");								/* wait 5 cycles to clear any pending interrupts */
   			asm ("      nop");
   			asm ("      nop");
   			asm ("      nop");
   			asm ("      nop");
   		
   			PieCtrlRegs.PIEACK.all = PIEACK_GROUP12;		/* Acknowledge the interrupts */
   			XIntruptRegs.XINT4CR.bit.ENABLE = 0;        	/* Disable XINT4 */
   			XIntruptRegs.XINT6CR.bit.ENABLE = 0;        	/* Disable XINT6 */
   			
   			/********* Calculate the change in orientation angle ********/

/* Change in Fibre Orientation = tan^(-1)⁡(□((Pixel Separation*Cylindrical Lens Ratio*Pixel Pitch)/(Distance Between Arrays)))*/
/* Where: */
/* 			pixel separation is the distance between peaks */
/*			Cylindrical Lens Ratio = 3 by virtue of the optics */
/*			Pixel Pitch =0.0125mm/pixel */
/*			Distance between arrays = 8.4mm */
/*	Fibre orientation is the angle between the fibre axis and the flow axis */

		Fibre_orient = atan(((Photo2_peak1 - Photo1_peak1) * 0.0375) * 0.11904); 
	if(Fibre_orient >= threshold_angle) 						/* Arbitary threshold set to 10 degrees */
	{
		Asb_fibre_count++; 										/* Increment asbestos fibre */
	}
	
   			file_number_int++;
   			if( file_number > 1140)					/* Each file is 14K. 2G SD capacity gives about 140 files */
   			{
   				file_number = 100;					/* Write over files if capacity reached */
   			}
   			file_number = file_number_int;
   			ltoa(file_number,file_number_str);
   			strcat(file_number_str,file_ext_str);
   			I2C_datetime_read();
   			fresult = f_mount(0, &Fatfs);
   			fresult = f_open(&File, file_number_str, FA_WRITE | FA_CREATE_ALWAYS);
    		if (fresult) die(fresult);   
    		WriteCount = f_printf(&File, "Alert, v%D,b%D\r\n",version,build);		
    		WriteCount = f_printf(&File, "Date,%D-%D-%D\r\n",date_uc,month_uc,year_uc);
    		WriteCount = f_printf(&File, "Time,%D:%D:%D\r\n",hours_uc,minutes_uc,seconds_uc);
    		WriteCount = f_printf(&File, "Pixel,Photo1,,Photo2\r\n");
    		WriteCount = f_printf(&File, ",1,2,1,2\r\n");
    		for(t=0;t<512;t++)
    		{
    			WriteCount = f_printf(&File,"%U,%U,%U,%U,%U\r\n",t,Photo1_Array1[t],Photo1_Array2[t],Photo2_Array1[t],Photo2_Array2[t]);
    		}
    		fresult = f_sync(&File);
    		fresult = f_close(&File);
    		f_mount(0, NULL);
   				ISR1_flag = FALSE;							/* flag set in Alert_Photo1_Trigger_isr1 */
   				ISR2_flag = FALSE;
   			PieCtrlRegs.PIEIER12.bit.INTx4 = 1;				/* Enable photo2_trigger interrupt */
   			PieCtrlRegs.PIEIER12.bit.INTx6 = 1;				/* Enable photo1_trigger interrupt */
   			XIntruptRegs.XINT4CR.bit.ENABLE = 1;        	/* Enable Xint4 */
   			XIntruptRegs.XINT6CR.bit.ENABLE = 1;        	/* Enable XINT6 */
   			EINT;											/* Enable gloabl interrupts */								
   		}   		
   		
   		EDIS;
   	}
} 	
//################################## END MAIN ####################################

// Step 7. Insert all local Interrupt Service Routines (ISRs) and functions here:	
interrupt void cpu_timer0_isr(void)
{
   CpuTimer0.InterruptCount++;								// increment timer0 counter every microsecond. 
   GpioDataRegs.GPBTOGGLE.bit.GPIO50 = 1;					// toggle Red alarm pin to check frequency
   PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;					// Acknowledge this interrupt to receive more interrupts from group 1
}


//******************************************************************************
//******************************************************************************
// Internal Functions
//******************************************************************************
//******************************************************************************

void scia_echoback_init()   
{   
    // Note: Clocks were turned on to the SCIA peripheral    
    // in the InitSysCtrl() function    
   
    SciaRegs.SCICCR.all =0x0007;   							// 1 stop bit,  No loopback    
                                   							// No parity,8 char bits,    
                                   							// async mode, idle-line protocol    
    SciaRegs.SCICTL1.all =0x0003;  							// enable TX, RX, internal SCICLK,    
                                   							// Disable RX ERR, SLEEP, TXWAKE    
    SciaRegs.SCICTL2.all =0x0003;   
    SciaRegs.SCICTL2.bit.TXINTENA =1;   
    SciaRegs.SCICTL2.bit.RXBKINTENA =1;   
    #if (CPU_FRQ_150MHZ)    
          SciaRegs.SCIHBAUD    =0x0000;  					// was 0x0001 for 9600 baud now 115000 baud @LSPCLK = 37.5MHz.    
          SciaRegs.SCILBAUD    =0x0028;  					// was 0x00E1 for 9600 baud 
    #endif    
    #if (CPU_FRQ_100MHZ)    
      SciaRegs.SCIHBAUD    =0x0001;  						// 9600 baud @LSPCLK = 20MHz.    
      SciaRegs.SCILBAUD    =0x0044;   
    #endif    
    SciaRegs.SCICTL1.all =0x0023;  							// Relinquish SCI from Reset    
}   
// Initialize the SCI FIFO    
void scia_fifo_init()   
{   
    SciaRegs.SCIFFTX.all=0xE040;   
    SciaRegs.SCIFFRX.all=0x204f;   
    SciaRegs.SCIFFCT.all=0x0;    
}   
//===========================================================================
// No more.
//===========================================================================
